from infi.clickhouse_orm.fields import (Field, StringField, FixedStringField, DateField, DateTimeField, BaseIntField,
                                        UInt8Field, UInt16Field, UInt32Field, UInt64Field, Int8Field, Int16Field,
                                        Int32Field, Int64Field, BaseFloatField, Float32Field, Float64Field,
                                        DecimalField, Decimal32Field, Decimal64Field, Decimal128Field, BaseEnumField,
                                        Enum8Field, Enum16Field, ArrayField, UUIDField, NullableField,
                                        LowCardinalityField)



from infi.clickhouse_orm.engines import (Engine, TinyLog, Memory, MergeTree, CollapsingMergeTree, SummingMergeTree,
                                         ReplacingMergeTree, Buffer, Merge, Distributed)


from infi.clickhouse_orm.query import (Operator, SimpleOperator, InOperator, LikeOperator, IExactOperator,
                                       NotOperator, BetweenOperator, register_operator, FOV, Q, QuerySet,
                                       AggregateQuerySet)


class AsyncQuerySet(QuerySet):
    async def execute(self):
        return await self._database.select(self.as_sql(), self._model_cls)

    async def count(self):
        if self._distinct or self._limits:
            sql = u'SELECT count() FROM (%s)' % self.as_sql()
            raw = await self._database.raw(sql)
            return int(raw[0][0])
        conditions = (self._where_q & self._prewhere_q).to_sql(self._model_cls)
        return await self._database.count(self._model_cls, conditions)



import pytz

from aioch import Client
from infi.clickhouse_orm.models import ModelBase
from infi.clickhouse_orm.database import Database, Page, DatabaseException, ServerError


class AsyncDatabase(Database):
    def __init__(self, db_name='default', db_host='127.0.0.1', db_port='9000',
                 username='default', password='', timeout=60):
        self.db_name = db_name
        self.db_host = db_host
        self.readonly = False
        self.timeout = timeout
        self.client = Client(host=db_host, port=db_port, database=db_name, user=username, password=password)
        self.settings = {}
        self.db_exists = False
        self.server_timezone = pytz.utc

    async def raw(self, query, settings=None, stream=False):
        '''
        Performs a query and returns its output as text.

        - `query`: the SQL query to execute.
        - `settings`: query settings to send as HTTP GET parameters
        - `stream`: if true, the HTTP response from ClickHouse will be streamed.
        '''
        query = self._substitute(query, None)
        return await self._send(query, settings=settings, stream=stream)

    async def count(self, model_class, conditions=None):
        '''
        Counts the number of records in the model's table.

        - `model_class`: the model to count.
        - `conditions`: optional SQL conditions (contents of the WHERE clause).
        '''
        query = 'SELECT count() FROM $table'
        if conditions:
            query += ' WHERE ' + conditions
        query = self._substitute(query, model_class)
        r = await self._send(query)
        return int(r[0][0])

    async def select(self, query, model_class=None, settings=None):
        '''
        Performs a query and returns a generator of model instances.

        - `query`: the SQL query to execute.
        - `model_class`: the model class matching the query's table,
          or `None` for getting back instances of an ad-hoc model.
        - `settings`: query settings to send as HTTP GET parameters
        '''
        query = self._substitute(query, model_class)
        lines, columns = await self._send(query, settings, with_column_types=True)
        field_names = [c[0] for c in columns]
        field_types = [c[1] for c in columns]
        model_class = model_class or ModelBase.create_ad_hoc_model(zip(field_names, field_types))
        return [model_class.from_list(line, field_names, self.server_timezone, self) for line in lines if line]

    async def _send(self, data, settings=None, stream=False, with_column_types=False):
        return await self.client.execute(data, settings, with_column_types=with_column_types)

    async def insert_async(self, model_instances, batch_size=1000):
        '''
        Insert records into the database.

        - `model_instances`: any iterable containing instances of a single model class.
        - `batch_size`: number of records to send per chunk (use a lower number if your records are very large).
        '''
        from six import next
        i = iter(model_instances)
        try:
            first_instance = next(i)
        except StopIteration:
            return  # model_instances is empty
        first_instance.set_database(self)
        model_class = first_instance.__class__

        if first_instance.is_read_only() or first_instance.is_system_model():
            raise DatabaseException("You can't insert into read only and system tables")

        fields_list = ','.join(
            ['`%s`' % name for name in first_instance.fields(writable=True)])

        def gen():
            values = list()
            values.append(first_instance.to_dict(include_readonly=False))
            # Collect lines in batches of batch_size
            lines = 2
            for instance in i:
                instance.set_database(self)
                values.append(instance.to_dict(include_readonly=False))
                lines += 1
                if lines >= batch_size:
                    # Return the current batch of lines
                    yield values
                    # Start a new batch
                    values = list()
                    lines = 0
            # Return any remaining lines in partial batch
            if lines:
                yield values

        for butch in gen():
            query = self._substitute('INSERT INTO $table (%s) VALUES ' % fields_list, model_class)
            await self._send(query, settings=butch)



import json
import datetime

from infi.clickhouse_orm.utils import (SPECIAL_CHARS, SPECIAL_CHARS_REGEX, escape, unescape, parse_tsv, parse_array,
                                       import_submodules, comma_join)
from .models import Model


class JSONEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, Model):
            return obj.to_dict()
        if isinstance(obj, (datetime.date, datetime.datetime)):
            return obj.isoformat()
        return json.JSONEncoder.default(self, obj)



import pytz

from six import iteritems
from infi.clickhouse_orm.models import ModelBase, Model as _Model, BufferModel, MergeModel, DistributedModel

from .query import AsyncQuerySet


class Model(_Model):
    @classmethod
    def objects_in_async(cls, database):
        return AsyncQuerySet(cls, database)

    @classmethod
    def from_list(cls, line, field_names, timezone_in_use=pytz.utc, database=None):
        from six import next
        values = iter(line)
        kwargs = {}
        for name in field_names:
            field = getattr(cls, name)
            kwargs[name] = field.to_python(next(values), timezone_in_use)

        obj = cls(**kwargs)
        if database is not None:
            obj.set_database(database)

        return obj



from setuptools import setup

README = '''
Async ClickHouse ORM.

More information available on read the doc: https://github.com/qvp/aiochorm
'''

setup(
    name='aiochorm',
    version='0.1.11',
    packages=['aiochorm'],
    description='Async ClickHouse ORM.',
    long_description=README,
    author='Alexander Kuzmenko',
    author_email='mail@alexanderkuzmenko.com',
    url='https://github.com/qvp/aiochorm',
    download_url='',
    license='MIT',
    install_requires=[
        'aioch==0.0.1',
        'infi.clickhouse-orm==1.2.0',
    ]
)
